@@ -238,15 +238,22 @@ for details.'''
                          '(e. g. "apt-get update" or adding apt sources); '
                          'can be a string with the commands, or a file '
                          'containing the commands')
-    # ensure that this fails with something other than 100, as apt-get update
-    # failures are usually transient
+    # Ensure that this fails with something other than 100 in most error cases,
+    # as apt-get update failures are usually transient; but if we find a
+    # nonexisting apt source (404 Not Found) we *do* want 100, as otherwise
+    # we'd get eternally looping tests.
     g_setup.add_argument('-U', '--apt-upgrade', dest='setup_commands',
                          action='append_const',
-                         const='(apt-get update || (sleep 15; apt-get update)'
-                         ' || (sleep 60; apt-get update) || false)'
+                         const='''(O=$(bash -o pipefail -ec 'apt-get update | tee /proc/self/fd/2') ||'''
+                         '{ [ "${O%404*Not Found*}" = "$O" ] || exit 100; sleep 15; apt-get update; }'''
+                         ' || { sleep 60; apt-get update; } || false)'
                          ' && $(which eatmydata || true) apt-get dist-upgrade -y -o '
                          'Dpkg::Options::="--force-confnew"',
                          help='Run apt update/dist-upgrade before the tests')
+    g_setup.add_argument('--setup-commands-boot', metavar='COMMANDS_OR_PATH',
+                         action='append', default=[],
+                         help='Run these commands after --setup-commands, '
+                         'and also every time the testbed is rebooted')
     g_setup.add_argument('--apt-pocket', action='append',
                          metavar='POCKETNAME[=pkgname,src:srcname,...]',
                          default=[],
@@ -376,6 +383,15 @@ for details.'''
             with open(c, encoding='UTF-8') as f:
                 args.setup_commands[i] = f.read().strip()
 
+    for i, c in enumerate(args.setup_commands_boot):
+        if '/' not in c:
+            shipped = os.path.join('/usr/share/autopkgtest/setup-commands', c)
+            if os.path.exists(shipped):
+                c = shipped
+        if os.path.exists(c):
+            with open(c, encoding='UTF-8') as f:
+                args.setup_commands_boot[i] = f.read().strip()
+
     # parse --copy arguments
     copy_pairs = []
     for arg in args.copy:
